@@ -25,6 +25,7 @@ from calibre.ebooks.chardet import xml_to_unicode
 from calibre.gui2 import NO_URL_FORMATTING, choose_files, error_dialog, gprefs
 from calibre.gui2.book_details import css
 from calibre.gui2.widgets import LineEditECM
+from calibre.gui2.widgets2 import to_plain_text
 from calibre.utils.config import tweaks
 from calibre.utils.imghdr import what
 from polyglot.builtins import filter, iteritems, itervalues, unicode_type
@@ -77,6 +78,7 @@ def lift_styles(tag, style_map):
 
 def filter_qt_styles(style):
     for k in tuple(style):
+        # -qt-paragraph-type is a hack used by Qt for empty paragraphs
         if k.startswith('-qt-'):
             del style[k]
 
@@ -191,6 +193,15 @@ def cleanup_qt_markup(root):
             use_implicit_styling_for_a(child, style_map)
         for child in tag.iterdescendants('span'):
             use_implicit_styling_for_span(child, style_map[child])
+        if tag.tag == 'p' and style_map[tag].get('-qt-paragraph-type') == 'empty':
+            del tag[:]
+            tag.text = '\xa0'
+        if tag.tag in ('ol', 'ul'):
+            for li in tag.iterdescendants('li'):
+                ts = style_map.get(li)
+                if ts:
+                    remove_margins(li, ts)
+                    remove_zero_indents(ts)
     for style in itervalues(style_map):
         filter_qt_styles(style)
     for tag, style in iteritems(style_map):
@@ -231,7 +242,7 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
     def __init__(self, parent=None):
         QTextEdit.__init__(self, parent)
         self.setTabChangesFocus(True)
-        self.document().setDefaultStyleSheet(css())
+        self.document().setDefaultStyleSheet(css() + '\n\nli { margin-top: 0.5ex; margin-bottom: 0.5ex; }')
         font = self.font()
         f = QFontInfo(font)
         delta = tweaks['change_book_details_font_size_by'] + 1
@@ -753,7 +764,13 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
                     with lopen(path, 'rb') as f:
                         data = f.read()
                 except EnvironmentError:
-                    pass
+                    if path.rpartition('.')[-1].lower() in {'jpg', 'jpeg', 'gif', 'png', 'bmp', 'webp'}:
+                        return QByteArray(bytearray.fromhex(
+                                    '89504e470d0a1a0a0000000d49484452'
+                                    '000000010000000108060000001f15c4'
+                                    '890000000a49444154789c6300010000'
+                                    '0500010d0a2db40000000049454e44ae'
+                                    '426082'))
                 else:
                     return QByteArray(data)
 
@@ -794,6 +811,8 @@ class EditorWidget(QTextEdit, LineEditECM):  # {{{
         if hasattr(parent, 'toolbars_visible'):
             vis = parent.toolbars_visible
             menu.addAction(_('%s toolbars') % (_('Hide') if vis else _('Show')), parent.toggle_toolbars)
+        menu.addSeparator()
+        menu.addAction(_('Smarten punctuation'), parent.smarten_punctuation)
         menu.exec_(ev.globalPos())
 
 # }}}
@@ -1053,7 +1072,7 @@ class Editor(QWidget):  # {{{
         tb.addWidget(self.toolbar3)
         l.addWidget(self.editor)
         self._layout.addWidget(self.tabs)
-        self.tabs.addTab(self.wyswyg, _('N&ormal view'))
+        self.tabs.addTab(self.wyswyg, _('&Normal view'))
         self.tabs.addTab(self.code_edit, _('&HTML source'))
         self.tabs.currentChanged[int].connect(self.change_tab)
         self.highlighter = Highlighter(self.code_edit.document())
@@ -1101,6 +1120,7 @@ class Editor(QWidget):  # {{{
         for x in ('bold', 'italic', 'underline', 'strikethrough'):
             ac = getattr(self.editor, 'action_'+x)
             self.toolbar3.addAction(ac)
+            self.addAction(ac)
         self.toolbar3.addSeparator()
 
         for x in ('left', 'center', 'right', 'justified'):
@@ -1134,7 +1154,7 @@ class Editor(QWidget):  # {{{
                 self.wyswyg_dirty = False
         elif index == 0:  # changing to wyswyg
             if self.source_dirty:
-                self.editor.html = unicode_type(self.code_edit.toPlainText())
+                self.editor.html = to_plain_text(self.code_edit)
                 self.source_dirty = False
 
     @property
@@ -1181,6 +1201,13 @@ class Editor(QWidget):  # {{{
     def hide_tabs(self):
         self.tabs.tabBar().setVisible(False)
 
+    def smarten_punctuation(self):
+        from calibre.ebooks.conversion.preprocess import smarten_punctuation
+        html = self.html
+        newhtml = smarten_punctuation(html)
+        if html != newhtml:
+            self.html = newhtml
+
 # }}}
 
 
@@ -1193,6 +1220,6 @@ if __name__ == '__main__':
     w.html = '''<h1>Test Heading</h1><blockquote>Test blockquote</blockquote><p><span style="background-color: rgb(0, 255, 255); ">He hadn't
     set <u>out</u> to have an <em>affair</em>, <span style="font-style:italic; background-color:red">
     much</span> less a <s>long-term</s>, <b>devoted</b> one.</span><p>hello'''
-    w.html = '<div><p id="moo">Testing <em>a</em> link.</p></div>'
+    w.html = '<div><p id="moo">Testing <em>a</em> link.</p><p>\xa0</p><p>ss</p></div>'
     app.exec_()
     # print w.html
